/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.cocoon.template.instruction;

import java.io.ByteArrayInputStream;
import java.util.Stack;

import org.apache.cocoon.core.xml.SAXParser;
import org.apache.cocoon.el.objectmodel.ObjectModel;
import org.apache.cocoon.el.parsing.Subst;
import org.apache.cocoon.template.environment.ExecutionContext;
import org.apache.cocoon.template.environment.ParsingContext;
import org.apache.cocoon.template.script.Invoker;
import org.apache.cocoon.template.script.event.Event;
import org.apache.cocoon.template.script.event.StartElement;
import org.apache.cocoon.xml.IncludeXMLConsumer;
import org.apache.cocoon.xml.XMLConsumer;
import org.apache.cocoon.xml.util.NamespacesTable;
import org.apache.commons.lang.BooleanUtils;
import org.apache.excalibur.xml.sax.XMLizable;
import org.w3c.dom.Node;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * @version SVN $Id: Out.java 587761 2007-10-24 03:08:05Z vgritsenko $
 */
public class Out extends Instruction {
    private final Subst compiledExpression;
    private Boolean xmlize;
    private Boolean stripRoot;

    public Out(ParsingContext parsingContext, StartElement raw, Attributes attrs, Stack stack) throws SAXException {

        super(raw);
        Locator locator = getLocation();
        String value = attrs.getValue("value");
        if (value == null)
            throw new SAXParseException("out: \"value\" is required", locator, null);

        this.compiledExpression = parsingContext.getStringTemplateParser().compileExpr(value, "out: \"value\": ",
                locator);
        String lenientValue = attrs.getValue("lenient");
        Boolean lenient = lenientValue == null ? null : Boolean.valueOf(lenientValue);

        // Why can out be lenient?
        if (lenient != null)
            this.compiledExpression.setLenient(lenient);

        String xmlize = attrs.getValue("xmlize");
        this.xmlize = (xmlize == null) ? null : Boolean.valueOf(xmlize);

        String stripRoot = attrs.getValue("strip-root");
        this.stripRoot = (stripRoot == null) ? null : Boolean.valueOf(stripRoot);
    }

    public Event execute(final XMLConsumer consumer, ObjectModel objectModel, ExecutionContext executionContext,
            MacroContext macroContext, NamespacesTable namespaces, Event startEvent, Event endEvent)
            throws SAXException {
        Object val;
        try {
            val = this.compiledExpression.getNode(objectModel);

            boolean stripRoot = BooleanUtils.toBoolean(this.stripRoot);
            // TODO: LG, I do not see a good way to do this.
            if (BooleanUtils.isTrue(this.xmlize)) {
                if (val instanceof Node || val instanceof Node[] || val instanceof XMLizable)
                    Invoker.executeNode(consumer, val, stripRoot);
                else {
                    SAXParser parser = executionContext.getSaxParser();
                    InputSource source = new InputSource(new ByteArrayInputStream(val.toString().getBytes()));
                    IncludeXMLConsumer includeConsumer = new IncludeXMLConsumer(consumer);
                    includeConsumer.setIgnoreRootElement(stripRoot);
                    parser.parse(source, includeConsumer);
                }
            } else
                Invoker.executeNode(consumer, val, stripRoot);
        } catch (Exception e) {
            throw new SAXParseException(e.getMessage(), getLocation(), e);
        }
        return getNext();
    }
}
